package com.os.ems.record.service.impl;

import java.text.SimpleDateFormat;
import java.util.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.os.ems.record.mapper.RecordIotenvInstantMapper;
import com.os.ems.record.domain.RecordIotenvInstant;
import com.os.ems.record.service.IRecordIotenvInstantService;

/**
 * 物联网数据Service业务层处理
 * 
 * @author zch
 * @date 2025-04-28
 */
@Service
public class RecordIotenvInstantServiceImpl implements IRecordIotenvInstantService 
{
    @Autowired
    private RecordIotenvInstantMapper recordIotenvInstantMapper;

    /**
     * 查询物联网数据
     * 
     * @param objid 物联网数据主键
     * @return 物联网数据
     */
    @Override
    public RecordIotenvInstant selectRecordIotenvInstantByObjid(Long objid)
    {
        return recordIotenvInstantMapper.selectRecordIotenvInstantByObjid(objid);
    }

    /**
     * 查询物联网数据列表
     *
     * @param recordIotenvInstant 物联网数据
     * @return 物联网数据
     */
//    @Override
//    public List<RecordIotenvInstant> selectRecordIotenvInstantList(RecordIotenvInstant recordIotenvInstant)
//    {
//        Map<String, Object> params = recordIotenvInstant.getParams();
//        return recordIotenvInstantMapper.selectRecordIotenvInstantList(recordIotenvInstant);
//    }
    @Override
    public List<RecordIotenvInstant> selectRecordIotenvInstantList(RecordIotenvInstant recordIotenvInstant) {

        Map<String, Object> params = recordIotenvInstant.getParams();
        // 检查是否有时间范围参数
        if (params != null && params.containsKey("beginRecordTime") && params.containsKey("endRecordTime")) {
            try {
                // 获取开始和结束时间
                String beginTimeStr = params.get("beginRecordTime").toString();
                String endTimeStr = params.get("endRecordTime").toString();

                // 解析日期
                SimpleDateFormat fullFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Date beginDate = fullFormat.parse(beginTimeStr);
                Date endDate = fullFormat.parse(endTimeStr);

                // 获取需要查询的表名列表
                List<String> tableNames = getTableNamesByDateRange(beginDate, endDate);
                if (tableNames.isEmpty()) {
                    return new ArrayList<>();
                }

                // 执行分表查询
                return recordIotenvInstantMapper.selectRecordIotenvInstantListFromTables(tableNames, recordIotenvInstant);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        // 如果没有时间范围或发生异常，则使用原始查询方法
        return recordIotenvInstantMapper.selectRecordIotenvInstantList(recordIotenvInstant);
    }

    /**
     * 新增物联网数据
     * 
     * @param recordIotenvInstant 物联网数据
     * @return 结果
     */
    @Override
    public int insertRecordIotenvInstant(RecordIotenvInstant recordIotenvInstant)
    {
        return recordIotenvInstantMapper.insertRecordIotenvInstant(recordIotenvInstant);
    }

    /**
     * 修改物联网数据
     * 
     * @param recordIotenvInstant 物联网数据
     * @return 结果
     */
    @Override
    public int updateRecordIotenvInstant(RecordIotenvInstant recordIotenvInstant)
    {
        return recordIotenvInstantMapper.updateRecordIotenvInstant(recordIotenvInstant);
    }

    /**
     * 批量删除物联网数据
     * 
     * @param objids 需要删除的物联网数据主键
     * @return 结果
     */
    @Override
    public int deleteRecordIotenvInstantByObjids(Long[] objids)
    {
        return recordIotenvInstantMapper.deleteRecordIotenvInstantByObjids(objids);
    }

    /**
     * 删除物联网数据信息
     * 
     * @param objid 物联网数据主键
     * @return 结果
     */
    @Override
    public int deleteRecordIotenvInstantByObjid(Long objid)
    {
        return recordIotenvInstantMapper.deleteRecordIotenvInstantByObjid(objid);
    }


    /**
     * 根据日期范围获取需要查询的表名列表
     *
     * @param beginDate 开始日期
     * @param endDate 结束日期
     * @return 表名列表
     */
    private List<String> getTableNamesByDateRange(Date beginDate, Date endDate) {
        List<String> tableNames = new ArrayList<>();

        // 如果开始日期晚于结束日期，则返回空列表
        if (beginDate.after(endDate)) {
            return tableNames;
        }

        // 设置日期格式
        SimpleDateFormat tableFormat = new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");

        try {
            // 获取开始日期和结束日期（不含时间部分）
            String beginDateStr = dateFormat.format(beginDate);
            String endDateStr = dateFormat.format(endDate);

            Date currentDate = dateFormat.parse(beginDateStr);
            Date endDateOnly = dateFormat.parse(endDateStr);

            // 循环添加每一天对应的表名
            Calendar calendar = Calendar.getInstance();
            while (!currentDate.after(endDateOnly)) {
                String tableSuffix = tableFormat.format(currentDate);
                String tableName = "record_iotenv_instant_" + tableSuffix;
                tableNames.add(tableName);

                // 日期加1天
                calendar.setTime(currentDate);
                calendar.add(Calendar.DAY_OF_MONTH, 1);
                currentDate = calendar.getTime();
            }
            for (String tableName : tableNames){
                if (!isTableExists(tableName)){
                    throw new SecurityException(tableName + "分表不存在");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return tableNames;
    }


    /**
     * 验证表是否存在
     *
     * @param tableName 表名
     * @return 是否存在
     */
    private boolean isTableExists(String tableName) {
        try {
            // 查询数据库中是否存在该表
            Map<String, Object> params = new HashMap<>();
            params.put("tableName", tableName);
            Integer count = recordIotenvInstantMapper.checkTableExists(params);
            return count != null && count > 0;
        } catch (Exception e) {
            return false;
        }
    }
}
